from django.db import models
from django.conf import settings

from django_pgjson.fields import JsonBField

from geokey.core.mixins import FilterMixin


class Subset(FilterMixin, models.Model):
    """
    Subsets define pre-defined query for a share of all data in a project.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)
    project = models.ForeignKey('projects.Project', related_name='subsets')
    filters = JsonBField(blank=True, null=True)
    where_clause = models.TextField(blank=True, null=True)



from django.test import TestCase
from django.core.urlresolvers import reverse, resolve

from ..views import (
    SubsetOverview,
    SubsetCreate,
    SubsetSettings,
    SubsetData,
    SubsetDelete
)


class UrlsTest(TestCase):
    def test_reverse_subset_list(self):
        self.assertEqual(
            reverse('admin:subset_list', args=(3, )),
            '/admin/projects/3/subsets/'
        )

    def test_resolve_subset_list(self):
        resolved = resolve('/admin/projects/3/subsets/')
        self.assertEqual(resolved.func.__name__, SubsetOverview.__name__)
        self.assertEqual(resolved.kwargs['project_id'], '3')

    def test_reverse_subset_create(self):
        self.assertEqual(
            reverse('admin:subset_create', args=(3, )),
            '/admin/projects/3/subsets/new/'
        )

    def test_resolve_subset_create(self):
        resolved = resolve('/admin/projects/3/subsets/new/')
        self.assertEqual(resolved.func.__name__, SubsetCreate.__name__)
        self.assertEqual(resolved.kwargs['project_id'], '3')

    def test_reverse_subset_settings(self):
        self.assertEqual(
            reverse('admin:subset_settings', args=(3, 5, )),
            '/admin/projects/3/subsets/5/'
        )

    def test_resolve_subset_settings(self):
        resolved = resolve('/admin/projects/3/subsets/5/')
        self.assertEqual(resolved.func.__name__, SubsetSettings.__name__)
        self.assertEqual(resolved.kwargs['project_id'], '3')
        self.assertEqual(resolved.kwargs['subset_id'], '5')

    def test_reverse_subset_delete(self):
        self.assertEqual(
            reverse('admin:subset_delete', args=(3, 5, )),
            '/admin/projects/3/subsets/5/delete/'
        )

    def test_resolve_subset_delete(self):
        resolved = resolve('/admin/projects/3/subsets/5/delete/')
        self.assertEqual(resolved.func.__name__, SubsetDelete.__name__)
        self.assertEqual(resolved.kwargs['project_id'], '3')
        self.assertEqual(resolved.kwargs['subset_id'], '5')

    def test_reverse_subset_data(self):
        self.assertEqual(
            reverse('admin:subset_data', args=(3, 5, )),
            '/admin/projects/3/subsets/5/data/'
        )

    def test_resolve_subset_data(self):
        resolved = resolve('/admin/projects/3/subsets/5/data/')
        self.assertEqual(resolved.func.__name__, SubsetData.__name__)
        self.assertEqual(resolved.kwargs['project_id'], '3')
        self.assertEqual(resolved.kwargs['subset_id'], '5')



from json import loads as json_loads

from django.views.generic import TemplateView
from django.utils.html import strip_tags
from django.shortcuts import redirect
from django.contrib import messages

from braces.views import LoginRequiredMixin

from geokey.core.decorators import handle_exceptions_for_admin
from geokey.projects.models import Project

from .models import Subset


class ProjectContext(object):
    """
    Mixin that provides the context to render templates. The context contains
    a Project instance based on project_id.
    """
    @handle_exceptions_for_admin
    def get_context_data(self, project_id, *args, **kwargs):
        """
        Returns the context containing the project instance.

        Parameters
        ----------
        project_id : int
            identifies the project in the data base

        Returns
        -------
        dict
        """
        project = Project.objects.as_admin(self.request.user, project_id)

        return super(ProjectContext, self).get_context_data(
            project=project,
            *args,
            **kwargs
        )


class SubsetOverview(LoginRequiredMixin, ProjectContext, TemplateView):
    """
    Displays the list of subsets in a project.
    """
    template_name = 'subsets/subsets_overview.html'


class SubsetCreate(LoginRequiredMixin, ProjectContext, TemplateView):
    """
    Provides the form to create a new subset.
    """
    template_name = 'subsets/subsets_create.html'

    def post(self, request, project_id):
        """
        Creates the subset based on the data entered by the user

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base

        Returns
        -------
        django.http.HttpResponseRedirect
            to SubsetData
        django.http.HttpResponse
            Rendered template, if an error occured
        """
        data = request.POST
        context = self.get_context_data(project_id)

        project = context.get('project')

        if project:
            subset = Subset.objects.create(
                name=strip_tags(data.get('name')),
                description=strip_tags(data.get('description')),
                creator=request.user,
                project=project
            )
            messages.success(self.request, "The subset has been created.")
            return redirect(
                'admin:subset_data',
                project_id=project.id,
                subset_id=subset.id
            )
        else:
            return self.render_to_response(context)


class SubsetContext(object):
    """
    Mixin that provides the context to render templates. The context contains
    a Subset instance based on project_id and subset_id.
    """
    @handle_exceptions_for_admin
    def get_context_data(self, project_id, subset_id, *args, **kwargs):
        """
        Returns the context containing the project instance.

        Parameters
        ----------
        project_id : int
            identifies the project in the data base
        subset_id : int
            identifies the subset in the data base

        Returns
        -------
        dict
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        subset = project.subsets.get(pk=subset_id)

        return super(SubsetContext, self).get_context_data(
            subset=subset,
            *args,
            **kwargs
        )


class SubsetSettings(LoginRequiredMixin, SubsetContext, TemplateView):
    """
    Provides the form to update the subset settings.
    """
    template_name = 'subsets/subsets_settings.html'

    def post(self, request, project_id, subset_id):
        """
        Updates the subset based on the data entered by the user

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base
        subset_id : int
            identifies the subset in the data base

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, subset_id)
        subset = context.get('subset')

        if subset:
            data = request.POST
            subset.name = strip_tags(data.get('name'))
            subset.description = strip_tags(data.get('description'))
            subset.save()

            messages.success(self.request, "The subset has been updated.")

        return self.render_to_response(context)


class SubsetData(LoginRequiredMixin, SubsetContext, TemplateView):
    """
    Provides the form to change the filter settings for the subset
    """
    template_name = 'subsets/subsets_data.html'

    def post(self, request, project_id, subset_id):
        """
        Updates the subset filter based on the data entered by the user

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base
        subset_id : int
            identifies the subset in the data base

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, subset_id)
        subset = context.get('subset')

        if subset:
            data = request.POST
            if data['filters'] != '-1':
                subset.filters = json_loads(data['filters'])
                subset.save()

            messages.success(self.request, "The subset has been updated.")

        return self.render_to_response(context)


class SubsetDelete(LoginRequiredMixin, SubsetContext, TemplateView):
    """
    Deletes a subset
    """
    template_name = 'base.html'

    def get(self, request, project_id, subset_id):
        """
        Deletes the subset

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the data base
        subset_id : int
            identifies the subset in the data base

        Returns
        -------
        django.http.HttpResponseRedirect
            to SubsetOverview
        django.http.HttpResponse
            Rendered template, if an error has occured
        """
        context = self.get_context_data(project_id, subset_id)
        subset = context.get('subset')

        if subset:
            subset.delete()
            messages.success(self.request, "The subset has been deleted.")
            return redirect('admin:subset_list', project_id=project_id)

        return self.render_to_response(context)



from django.test import TestCase
from ..templatetags import count


class TemplateTagTest(TestCase):
    def test_more_link_text(self):
        self.assertEqual(
            count.more_link_text(6, 'category', 'categories'),
            'Show 1 more category'
        )

        self.assertEqual(
            count.more_link_text(7, 'category', 'categories'),
            'Show 2 more categories'
        )

        self.assertEqual(
            count.more_link_text(5, 'category', 'categories', 4),
            'Show 1 more category'
        )

        self.assertEqual(
            count.more_link_text(6, 'category', 'categories', 4),
            'Show 2 more categories'
        )
